import click
from convisoappsec.flowcli import help_option
from .assert_security_rules import assert_security_rules
from .run import run


@click.group()
@help_option
def vulnerability():
    pass


vulnerability.add_command(assert_security_rules)
vulnerability.add_command(run)

vulnerability.epilog = '''
  Run flow vulnerability COMMAND --help for more information on a command.
'''
